@@ -9,8 +9,8 @@ def breaks_constraints(moves: List["Move"]) -> bool:
     for move in moves:
         if breaks_practice_constraint(move):
             return True
-        if move.player.lock:
-            return True
+        # if move.player.lock:
+        #     return True
     return False
 
 
